@@ -23,8 +23,9 @@ class DAS_network(ANN):
 
     def __init__(self, X, y, d, alpha=0.001, decay_rate=1.0, decay_step=10**5,
                  beta1=0.9, beta2=0.999, lamb=0.0, n_out=1,
-                 param_specific_learn_rate=True, loss='squared', activation='tanh',
-                 activation_out='linear', n_softmax=0, n_layers=2, n_neurons=16,
+                 param_specific_learn_rate=True, loss='squared',
+                 activation='tanh', activation_out='linear', activation_das='linear',
+                 n_softmax=0, n_layers=2, n_neurons=16,
                  bias=True, batch_size=1, save=True,
                  name='DAS', on_gpu=False,
                  standardize_X=True, standardize_y=True, **kwargs):
@@ -99,6 +100,9 @@ class DAS_network(ANN):
         # the dimension of the active subspace
         self.d = d
 
+        # the activation of the DAS layer
+        self.activation_das = activation_das
+
         # set all the common parameters via the parent ANN class,
         # but overwrite the init_network subsroutine in this class
         super().__init__(X, y, alpha=alpha, decay_rate=decay_rate,
@@ -127,22 +131,33 @@ class DAS_network(ANN):
         self.layers = []
 
         n_hidden = self.n_layers
-        if 'cumsum' in kwargs and kwargs['cumsum']:
-            self.n_layers += 1
 
         # add the input layer
         self.layers.append(Layer(self.n_in, 0, self.n_layers, 'linear',
                                  self.loss, False, batch_size=self.batch_size,
                                  lamb=self.lamb, on_gpu=self.on_gpu))
 
+        # by default, the 1st layer does not have a bias neuron. This way
+        # the orthogonal vectors are only related to the D inputs, and not the
+        # D+1 inputs + 1 bias neuron, which does not make physical sense
+        self.bias[0] = False
+
         # add the deep active subspace layer
-        self.layers.append(DAS_Layer(self.d, self.n_layers, True,
-                                     batch_size=self.batch_size))
+        self.layers.append(
+            DAS_Layer(
+                self.d,
+                self.n_layers,
+                self.bias[1],
+                activation=self.activation_das,
+                batch_size=self.batch_size))
+
+        # adjust layer_activation of the ANN superclass
+        self.layer_activation[1] = self.activation_das
 
         # add the hidden layers
         for r in range(2, n_hidden):
-            self.layers.append(Layer(self.n_neurons, r, self.n_layers, self.activation,
-                                     self.loss, self.bias, batch_size=self.batch_size,
+            self.layers.append(Layer(self.n_neurons, r, self.n_layers, self.layer_activation[r],
+                                     self.loss, self.bias[r], batch_size=self.batch_size,
                                      lamb=self.lamb, on_gpu=self.on_gpu))
 
         # add the output layer
@@ -159,101 +174,107 @@ class DAS_network(ANN):
                 on_gpu=self.on_gpu,
                 **kwargs))
 
-        super().connect_layers()
-        super().print_network_info()
+        # super().connect_layers()
+        # super().print_network_info()
 
-    def feed_forward(self, X_i, batch_size=1):
-        """
-        Run the deep active subspace network forward.
+    # I REMOVED THIS AS IT IS NOT STRICTLY NECESSARY, THE ONLY DIFFERENCE
+    # IS THAT THE ANN FEED_FORWARD SUBROUTINE CAN HAVE A BIAS. THE LAYER
+    # BEFORE THE DAS LAYER DOES NOT HAVE A BIAS, WHICH WAS TYPICALLY THE
+    # INPUT LAYER. MAYBE IT CAN HAVE A BIAS, ALTHOUGH IT PROBABLY SHOULDN'T
+    # HAVE ONE BY DEFAULT. CHECK OUT.
+    # def feed_forward(self, X_i, batch_size=1):
+    #     """
+    #     Run the deep active subspace network forward.
 
-        Parameters
-        ----------
-        X_i : array
-            The feauture array, needs to have shape [batch size, number of features].
-        batch_size : int, optional
-            The bath size. The default is 1.
+    #     Parameters
+    #     ----------
+    #     X_i : array
+    #         The feauture array, needs to have shape [batch size, number of features].
+    #     batch_size : int, optional
+    #         The bath size. The default is 1.
 
-        Returns
-        -------
-        array
-            The prediction of the deep active subspace neural network.
-        """
+    #     Returns
+    #     -------
+    #     array
+    #         The prediction of the deep active subspace neural network.
+    #     """
 
-        # set the features at the output of in the input layer
-        self.layers[0].h = X_i.T
+    #     # set the features at the output of in the input layer
+    #     self.layers[0].h = X_i.T
 
-        for i in range(1, self.n_layers + 1):
-            # compute the output on the layer using matrix-maxtrix multiplication
-            self.layers[i].compute_output(batch_size)
+    #     for i in range(1, self.n_layers + 1):
+    #         # compute the output on the layer using matrix-maxtrix multiplication
+    #         self.layers[i].compute_output(batch_size)
 
-        return self.layers[-1].h
+    #     return self.layers[-1].h
 
-    def batch(self, X_i, y_i, alpha=0.001, beta1=0.9, beta2=0.999):
-        """
-        Update the weights of the neural network and the weights of
-        the Gram-Schmidt vectors using a mini batch.
+    # I MOVED THIS TOO TO THE ANN CLASS
+    # def batch(self, X_i, y_i, alpha=0.001, beta1=0.9, beta2=0.999):
+    #     """
+    #     Update the weights of the neural network and the weights of
+    #     the Gram-Schmidt vectors using a mini batch.
 
-        Parameters
-        ----------
-        X_i : array
-            The input features of the mini batch.
-        y_i : array
-            The target data of the mini batch.
-        alpha : float, optional
-            The learning rate. The default is 0.001.
-        beta1 : float, optional
-            Momentum parameter controlling the moving average of the loss gradient.
-            Used for the parameter-specific learning rate. The default is 0.9.
-        beta2 : float, optional
-            Parameter controlling the moving average of the squared gradient.
-            Used for the parameter-specific learning rate. The default is 0.999.
+    #     Parameters
+    #     ----------
+    #     X_i : array
+    #         The input features of the mini batch.
+    #     y_i : array
+    #         The target data of the mini batch.
+    #     alpha : float, optional
+    #         The learning rate. The default is 0.001.
+    #     beta1 : float, optional
+    #         Momentum parameter controlling the moving average of the loss gradient.
+    #         Used for the parameter-specific learning rate. The default is 0.9.
+    #     beta2 : float, optional
+    #         Parameter controlling the moving average of the squared gradient.
+    #         Used for the parameter-specific learning rate. The default is 0.999.
 
-        Returns
-        -------
-        None.
+    #     Returns
+    #     -------
+    #     None.
 
-        """
+    #     """
 
-        self.feed_forward(X_i, self.batch_size)
-        self.back_prop(y_i)
+    #     self.feed_forward(X_i, self.batch_size)
+    #     self.back_prop(y_i)
 
-        for r in range(1, self.n_layers + 1):
+    #     for r in range(1, self.n_layers + 1):
 
-            layer_r = self.layers[r]
+    #         layer_r = self.layers[r]
 
-            # Deep active subspace layer
-            if r == 1:
-                # momentum
-                layer_r.V = beta1 * layer_r.V + (1.0 - beta1) * layer_r.L_grad_Q
-                # moving average of squared gradient magnitude
-                layer_r.A = beta2 * layer_r.A + (1.0 - beta2) * layer_r.L_grad_Q**2
-            # standard layer
-            else:
-                # momentum
-                layer_r.V = beta1 * layer_r.V + (1.0 - beta1) * layer_r.L_grad_W
-                # moving average of squared gradient magnitude
-                layer_r.A = beta2 * layer_r.A + (1.0 - beta2) * layer_r.L_grad_W**2
+    #         # Deep active subspace layer
+    #         if isinstance(layer_r, DAS_Layer):
+    #             # momentum
+    #             layer_r.V = beta1 * layer_r.V + (1.0 - beta1) * layer_r.L_grad_Q
+    #             # moving average of squared gradient magnitude
+    #             layer_r.A = beta2 * layer_r.A + (1.0 - beta2) * layer_r.L_grad_Q**2
+    #         # standard layer
+    #         else:
+    #             # momentum
+    #             layer_r.V = beta1 * layer_r.V + (1.0 - beta1) * layer_r.L_grad_W
+    #             # moving average of squared gradient magnitude
+    #             layer_r.A = beta2 * layer_r.A + (1.0 - beta2) * layer_r.L_grad_W**2
 
-            # select learning rate
-            if not self.param_specific_learn_rate:
-                # same alpha for all weights
-                alpha_i = alpha
-            # param specific learning rate
-            else:
-                # RMSProp
-                alpha_i = alpha / (np.sqrt(layer_r.A + 1e-8))
+    #         # select learning rate
+    #         if not self.param_specific_learn_rate:
+    #             # same alpha for all weights
+    #             alpha_i = alpha
+    #         # param specific learning rate
+    #         else:
+    #             # RMSProp
+    #             alpha_i = alpha / (np.sqrt(layer_r.A + 1e-8))
 
-            # gradient descent update step with L2 regularization
-            if self.lamb > 0.0:
-                layer_r.W = (1.0 - layer_r.Lamb * alpha_i) * layer_r.W - alpha_i * layer_r.V
-            # without regularization
-            else:
-                # Deep active subspace layer
-                if r == 1:
-                    # update the Q weights
-                    layer_r.Q = layer_r.Q - alpha_i * layer_r.V
-                    # compute the weights W(Q) via Gram Schmidt
-                    layer_r.compute_weights()
-                # standard layer
-                else:
-                    layer_r.W = layer_r.W - alpha_i * layer_r.V
+    #         # gradient descent update step with L2 regularization
+    #         if self.lamb > 0.0:
+    #             layer_r.W = (1.0 - layer_r.Lamb * alpha_i) * layer_r.W - alpha_i * layer_r.V
+    #         # without regularization
+    #         else:
+    #             # Deep active subspace layer
+    #             if isinstance(layer_r, DAS_Layer):
+    #                 # update the Q weights
+    #                 layer_r.Q = layer_r.Q - alpha_i * layer_r.V
+    #                 # compute the weights W(Q) via Gram Schmidt
+    #                 layer_r.compute_weights()
+    #             # standard layer
+    #             else:
+    #                 layer_r.W = layer_r.W - alpha_i * layer_r.V
