@@ -3,7 +3,7 @@ Class for a neural network Layer.
 """
 import sys
 import numpy as np
-from scipy.stats import norm
+from scipy.stats import norm, bernoulli
 
 
 class Layer:
@@ -32,10 +32,10 @@ class Layer:
         activation : string, optional
             The name of the activation function of the hidden layers.
             The default is 'tanh'.
-        loss : string, optional
-            The name of the loss function. The default is 'squared'.
+        loss : string
+            The name of the loss function.
         bias : boolean, optional
-            Use a bias neuron. The default is True.
+            Use a bias neuron. The default is False.
         batch_size : int, optional
             The size of the mini batch. The default is 1.
         lamb : float, optional
@@ -154,7 +154,7 @@ class Layer:
         if self.bias:
             self.Lamb[-1, :] = 0.0
 
-    def compute_output(self, batch_size):
+    def compute_output(self, batch_size, dropout = False, **kwargs):
         """
         Compute the output of the current layer in one shot using matrix -
         vector/matrix multiplication.
@@ -207,6 +207,10 @@ class Layer:
             print('Unknown activation type')
             sys.exit()
 
+        if dropout:
+            r = bernoulli.rvs(kwargs['dropout_prob'], size = self.h.shape)
+            self.h *= r
+
         # add bias neuron output
         if self.bias:
             self.h = np.vstack([self.h, np.ones(batch_size)])
@@ -345,7 +349,14 @@ class Layer:
                 # Using this computes the derivatives of the L2^2 norm of y
                 # self.delta_hy = 2 * self.h
                 # Using this computes the derivatives of the L2^2 norm of y
-                self.delta_hy = self.h / np.linalg.norm(self.h)
+                if self.loss == 'cross_entropy':
+                    o_i = []
+                    [o_i.append(np.exp(h_i) / np.sum(np.exp(h_i), axis=0))
+                     for h_i in np.split(self.h, self.n_softmax)]
+                    o_i = np.concatenate(o_i)
+                    self.delta_hy = o_i / np.linalg.norm(o_i)
+                else:
+                    self.delta_hy = self.h / np.linalg.norm(self.h)
         else:
             # get the delta_ho values of the next layer (layer r+1)
             delta_hy_rp1 = self.layer_rp1.delta_hy
@@ -395,9 +406,16 @@ class Layer:
 
             # for multinomial classification
             elif self.loss == 'cross_entropy':
-
-                # (see eq. 3.22 of Aggarwal book)
-                self.delta_ho = self.o_i - y_i
+                # one-hot encoded data (y_i contains only 0's and 1's)
+                if np.array_equal(y_i, y_i.astype(bool)):
+                    # (see eq. 3.22 of Aggarwal book)
+                    self.delta_ho = self.o_i - y_i
+                # y_i is a more general probability mass function
+                # delta_ho_i = sum_j(y_j * o_i) - y_i
+                else:
+                    self.delta_ho = np.array([(self.o_i[i] * y_i).sum(axis=0) 
+                                              for i in range(y_i.shape[0])])
+                    self.delta_ho -= y_i
 
             elif self.loss == 'kernel_mixture' and self.n_softmax > 0:
 
@@ -452,7 +470,7 @@ class Layer:
         """
         h_rm1 = self.layer_rm1.h
         delta_ho_grad_Phi = self.delta_ho * self.grad_Phi
-        self.L_grad_W = np.dot(h_rm1, delta_ho_grad_Phi.T)
+        self.L_grad_W = np.dot(h_rm1, delta_ho_grad_Phi.T)# / self.batch_size
 
     def back_prop(self, y_i):
         """
