@@ -3,13 +3,20 @@ Class for an artificial neural network.
 """
 
 import pickle
-import tkinter as tk
-from tkinter import filedialog
 import numpy as np
-# from scipy.stats import rv_discrete
-# import h5py
+from scipy.stats import bernoulli
+from tqdm import tqdm
+
+try:
+    import tkinter as tk
+    from tkinter import filedialog
+except ImportError:
+    print("WARNING: tkinter not imported")
+    print("save_ANN and load_ANN have no graphical support, \
+          use these by specifying file_path=")
 
 from .Layer import Layer
+from .DAS_Layer import DAS_Layer
 
 
 class ANN:
@@ -21,7 +28,7 @@ class ANN:
                  beta2=0.999, lamb=0.0, n_out=1, loss='squared', activation='tanh',
                  activation_out='linear', n_softmax=0, n_layers=2, n_neurons=16,
                  bias=True, batch_size=1, param_specific_learn_rate=True,
-                 save=True, on_gpu=False, name='ANN',
+                 save=False, on_gpu=False, name='ANN',
                  standardize_X=True, standardize_y=True, **kwargs):
         """
         Initialize the Artificial Neural Network object.
@@ -52,7 +59,7 @@ class ANN:
             The number of output neurons. The default is 1.
         loss : string, optional
             The name of the loss function. The default is 'squared'.
-        activation : string, optional
+        activation : string or list of strings, optional
             The name of the activation function of the hidden layers.
             The default is 'tanh'.
         activation_out : string, optional
@@ -60,7 +67,7 @@ class ANN:
             The default is 'linear'.
         n_softmax : int, optional
             The number of softmax layers attached to the output. The default is 0.
-        n_layers : int, optional
+        n_layers : integer or list of integers, optional
             The number of layers, not counting the input layer. The default is 2.
         n_neurons : int, optional
             The number of neurons per hidden layer. The default is 16.
@@ -72,7 +79,7 @@ class ANN:
             Use parameter-specific learing rate. The default is True.
         save : boolean, optional
             Save the neural network to a pickle file after training.
-            The default is True.
+            The default is False.
         on_gpu : boolean, optional
             Train the neural network on a GPU using cupy. NOT IMPLEMENTED IN THIS VERSION.
             The default is False.
@@ -104,6 +111,9 @@ class ANN:
         except IndexError:
             self.n_in = 1
 
+        # number of output neurons
+        self.n_out = n_out
+
         # #use either numpy or cupy via xp based on the on_gpu flag
         # global xp
         # if on_gpu == False:
@@ -128,27 +138,124 @@ class ANN:
         self.standardize_X = standardize_X
         self.standardize_y = standardize_y
 
-        # number of layers (hidden + output)
-        self.n_layers = n_layers
+        # size of the mini batch used in stochastic gradient descent
+        self.batch_size = batch_size
+        
+        # set dropout to False, can be changed in train subroutine
+        self.dropout = False
 
-        # number of neurons in a hidden layer
-        self.n_neurons = n_neurons
+        ###########################################
 
-        # number of output neurons
-        self.n_out = n_out
+        # use a user-specified list of layer objects to create the ANN
+        if 'layers' in kwargs:
+            # user-specified layers
+            self.layers = kwargs['layers']
 
-        # use bias neurons
-        self.bias = bias
+            assert isinstance(self.layers, list), "layers must be stored in a list"
 
-        # loss function type
-        self.loss = loss
+            # number of layers (hidden + output)
+            self.n_layers = len(self.layers) - 1
+
+            # number of neurons in the hidden layers
+            self.n_neurons = [layer.n_neurons for layer in self.layers[1:-1]]
+
+            # bias per layer
+            self.bias = [layer.bias for layer in self.layers]
+
+            # loss function
+            self.loss = self.layers[-1].loss
+
+            # activation function per hidden layer
+            self.activation = [layer.activation for layer in self.layers[1:-1]]
+
+            # activation of the output layer
+            self.activation_out = self.layers[-1].activation
+
+            # L2 regularization parameter
+            self.lamb = self.layers[-1].lamb
+
+            # number of softmax layers at the output
+            self.n_softmax = self.layers[-1].n_softmax
+
+            self.set_batch_size(self.batch_size)
+
+        # the default option, create the layers using the parameters
+        # of this subroutine
+        else:
+            self.layers = []
+
+            # number of layers (hidden + output)
+            self.n_layers = n_layers
+
+            self.n_neurons = n_neurons
+
+            # use bias neurons
+            if isinstance(bias, bool):
+                self.bias = []
+                for i in range(n_layers):
+                    self.bias.append(bias)
+            else:
+                self.bias = bias
+
+            # loss function type
+            self.loss = loss
+
+            # activation function of the hidden layers
+            self.activation = activation
+
+            # activation function of the output layer
+            self.activation_out = activation_out
+
+            # L2 regularization parameter
+            self.lamb = lamb
+
+            # number of sofmax layers
+            self.n_softmax = n_softmax
+
+        ############################################
+
+        # bias type checking
+        assert isinstance(
+            bias, bool) or isinstance(
+            bias, list), "bias must be boolean or a list of boolan"
+
+        # type checking the number of neurons
+        assert isinstance(self.n_neurons, int) or isinstance(self.n_neurons, list), \
+            "n_neurons must be a list or an integer"
+
+        # list of all layer sizes
+        self.layer_sizes = [self.n_in]
+
+        # constant size hidden layer
+        if isinstance(self.n_neurons, int):
+            for i in range(n_layers - 1):
+                self.layer_sizes.append(self.n_neurons)
+        # variable user-specified size hidden layers
+        else:
+            for i in range(n_layers - 1):
+                self.layer_sizes.append(self.n_neurons[i])
+
+        self.layer_sizes.append(n_out)
+
+        # type checking the activation
+        assert isinstance(activation, str) or isinstance(activation, list), \
+            "activation must be a string or a list of strings"
+
+        # list of activation functions per layer
+        self.layer_activation = ['linear']
+        # same activation for each hidden layer
+        if isinstance(self.activation, str):
+            for i in range(self.n_layers - 1):
+                self.layer_activation.append(self.activation)
+        else:
+            for i in range(self.n_layers - 1):
+                self.layer_activation.append(self.activation[i])
+
+        self.layer_activation.append(self.activation_out)
 
         # training rate
         self.alpha = alpha
 
-        # L2 regularization parameter
-        self.lamb = lamb
-
         # the rate of decay and decay step for alpha
         self.decay_rate = decay_rate
         self.decay_step = decay_step
@@ -162,15 +269,6 @@ class ANN:
         # use parameter specific learning rate
         self.param_specific_learn_rate = param_specific_learn_rate
 
-        # activation function of the hidden layers
-        self.activation = activation
-
-        # activation function of the output layer
-        self.activation_out = activation_out
-
-        # number of sofmax layers
-        self.n_softmax = n_softmax
-
         # save the neural network after training
         self.save = save
         self.name = name
@@ -178,12 +276,18 @@ class ANN:
         # additional variables/dicts etc that must be stored in the ann object
         self.aux_vars = kwargs
 
-        # size of the mini batch used in stochastic gradient descent
-        self.batch_size = batch_size
-
         self.loss_vals = []
 
-        self.init_network(**kwargs)
+        # initialize network using standard setting if no layers
+        # are specified
+        if 'layers' not in kwargs:
+            self.init_network(**kwargs)
+
+        # connect each layer with its neighbours
+        self.connect_layers()
+
+        # print some network stats to screen
+        self.print_network_info()
 
     def init_network(self, **kwargs):
         """
@@ -196,18 +300,19 @@ class ANN:
 
         """
 
-        self.layers = []
-
-        # add the input layer
-        self.layers.append(Layer(self.n_in, 0, self.n_layers, 'linear',
-                                 self.loss, self.bias, batch_size=self.batch_size,
-                                 lamb=self.lamb, on_gpu=self.on_gpu))
-
-        # add the hidden layers
-        for r in range(1, self.n_layers):
-            self.layers.append(Layer(self.n_neurons, r, self.n_layers, self.activation,
-                                     self.loss, self.bias, batch_size=self.batch_size,
-                                     lamb=self.lamb, on_gpu=self.on_gpu))
+        # add the inputs and hidden layers
+        for r in range(self.n_layers):
+            self.layers.append(
+                Layer(
+                    self.layer_sizes[r],
+                    r,
+                    self.n_layers,
+                    self.layer_activation[r],
+                    self.loss,
+                    self.bias[r],
+                    batch_size=self.batch_size,
+                    lamb=self.lamb,
+                    on_gpu=self.on_gpu))
 
         # add the output layer
         self.layers.append(
@@ -217,15 +322,14 @@ class ANN:
                 self.n_layers,
                 self.activation_out,
                 self.loss,
+                bias=False,
                 batch_size=self.batch_size,
                 lamb=self.lamb,
                 n_softmax=self.n_softmax,
                 on_gpu=self.on_gpu,
                 **kwargs))
 
-        self.connect_layers()
-
-        self.print_network_info()
+        # self.connect_layers()
 
     def connect_layers(self):
         """
@@ -262,15 +366,27 @@ class ANN:
         """
 
         # set the features at the output of in the input layer
-        if not self.bias:
+        if not self.bias[0]:
             self.layers[0].h = X_i.T
         else:
             self.layers[0].h = np.ones([self.n_in + 1, batch_size])
             self.layers[0].h[0:self.n_in, :] = X_i.T
+        
+        # apply dropout to the input layer
+        if self.dropout:
+            r = bernoulli.rvs(self.dropout_prob[0], size=X_i.T.shape)
+            self.layers[0].h[0:self.n_in, :] *= r
 
-        for i in range(1, self.n_layers + 1):
+        for i in range(1, self.n_layers):
             # compute the output on the layer using matrix-maxtrix multiplication
-            self.layers[i].compute_output(batch_size)
+            if self.dropout: # with dropout
+                self.layers[i].compute_output(batch_size, dropout=self.dropout,
+                                              dropout_prob = self.dropout_prob[i])
+            else: # without
+                self.layers[i].compute_output(batch_size)
+                
+        # output layer, never use dropout here
+        self.layers[i + 1].compute_output(batch_size)
 
         return self.layers[-1].h
 
@@ -315,7 +431,7 @@ class ANN:
         # return values and index of highest probability and random samples from pmf
         return probs, idx_max, None
 
-    def d_norm_y_dX(self, X_i, batch_size=1, feed_forward=True, norm = True):
+    def d_norm_y_dX(self, X_i, batch_size=1, feed_forward=True, norm=True, layer_idx=0):
         """
         Compute the derivatives of the squared L2 norm of the output wrt
         the inputs.
@@ -332,6 +448,8 @@ class ANN:
             Compute the gradient of ||y||_2. If False it computes the gradient of
             y, if y is a scalar. If False and y is a vector, the resulting gradient is the
             column sum of the full Jacobian matrix.
+        layer_idx : int, optional, default is 0.
+            Index for the layer of which to return the derivative. Default is 0, the input layer.
 
         Returns
         -------
@@ -348,8 +466,8 @@ class ANN:
             if i > 0:
                 self.layers[i].compute_y_grad_W()
 
-        # delta_hy of the input layer = the derivative of the normed output
-        return self.layers[0].delta_hy
+        # delta_hy of the (input) layer = the derivative of the normed output
+        return self.layers[layer_idx].delta_hy
 
     def back_prop(self, y_i):
         """
@@ -371,10 +489,14 @@ class ANN:
         for i in range(self.n_layers, 0, -1):
             self.layers[i].back_prop(y_i)
 
-    def batch(self, X_i, y_i, alpha=0.001, beta1=0.9, beta2=0.999):
+    def batch(self, X_i, y_i, alpha=0.001, beta1=0.9, beta2=0.999, **kwargs):
         """
         Update the weights using a mini batch.
 
+        In the case of a deep-active subspace layer, update the weights of the
+        neural network and the weights of the Gram-Schmidt vectors using a
+        mini batch.
+
         Parameters
         ----------
         X_i : array
@@ -395,6 +517,7 @@ class ANN:
         None.
 
         """
+        
         self.feed_forward(X_i, self.batch_size)
         self.back_prop(y_i)
 
@@ -402,10 +525,18 @@ class ANN:
 
             layer_r = self.layers[r]
 
-            # momentum
-            layer_r.V = beta1 * layer_r.V + (1.0 - beta1) * layer_r.L_grad_W
-            # moving average of squared gradient magnitude
-            layer_r.A = beta2 * layer_r.A + (1.0 - beta2) * layer_r.L_grad_W**2
+            # Deep active subspace layer
+            if isinstance(layer_r, DAS_Layer):
+                # momentum
+                layer_r.V = beta1 * layer_r.V + (1.0 - beta1) * layer_r.L_grad_Q
+                # moving average of squared gradient magnitude
+                layer_r.A = beta2 * layer_r.A + (1.0 - beta2) * layer_r.L_grad_Q**2
+            # standard layer
+            else:
+                # momentum
+                layer_r.V = beta1 * layer_r.V + (1.0 - beta1) * layer_r.L_grad_W
+                # moving average of squared gradient magnitude
+                layer_r.A = beta2 * layer_r.A + (1.0 - beta2) * layer_r.L_grad_W**2
 
             # select learning rate
             if not self.param_specific_learn_rate:
@@ -416,27 +547,28 @@ class ANN:
                 # RMSProp
                 alpha_i = alpha / (np.sqrt(layer_r.A + 1e-8))
 
-                # Adam
-                #alpha_t = alpha*np.sqrt(1.0 - beta2**t)/(1.0 - beta1**t)
-                #alpha_i = alpha_t/(np.sqrt(layer_r.A + 1e-8))
-
             # gradient descent update step with L2 regularization
             if self.lamb > 0.0:
                 layer_r.W = (1.0 - layer_r.Lamb * alpha_i) * layer_r.W - alpha_i * layer_r.V
             # without regularization
             else:
-                layer_r.W = layer_r.W - alpha_i * layer_r.V
+                # Deep active subspace layer
+                if isinstance(layer_r, DAS_Layer):
+                    # update the Q weights
+                    layer_r.Q = layer_r.Q - alpha_i * layer_r.V
+                    # compute the weights W(Q) via Gram Schmidt
+                    layer_r.compute_weights()
+                # standard layer
+                else:
+                    layer_r.W = layer_r.W - alpha_i * layer_r.V
 
-                # Nesterov momentum
-                # layer_r.W += -alpha*beta1*layer_r.V
-
-    # train the neural network
     def train(
             self,
             n_batch,
-            store_loss=False,
+            store_loss=True,
             sequential=False,
-            verbose=True):
+            verbose=True,
+            dropout=False, **kwargs):
         """
         Train the neural network using stochastic gradient descent.
 
@@ -445,12 +577,18 @@ class ANN:
         n_batch : int
             The number of mini-batch iterations.
         store_loss : boolean, optional
-            Store the values of the loss function. The default is False.
+            Store the values of the loss function. The default is True.
         sequential : boolean, optional
             Sample a sequential slab of data, starting from a random point.
             The default is False.
         verbose : boolean, optional
-            Print information to screen while training. The default is True.
+            Print information to screen while training. The default is False.
+        dropout : boolean, optional
+            Use dropout regularization. The default is False. To manually
+            specify the dropout probabilities, specify the keyword argument
+            "dropout_prob", as a list of probabilities of retaining neurons
+            per layer. Otherwise, 0.8 is used for the input layer, 
+            and 0.5 for the hidden layers.
 
         Returns
         -------
@@ -458,7 +596,19 @@ class ANN:
 
         """
 
-        for i in range(n_batch):
+        if dropout:
+            self.dropout = dropout
+            # use standard dropout probabilities
+            if 'dropout_prob' not in kwargs:
+                self.dropout_prob = [0.8]
+                for i in range(self.n_layers - 1):
+                    self.dropout_prob.append(0.5)
+            # user-specified dropout probabilities
+            else:
+                self.dropout_prob = kwargs['dropout_prob']
+
+        # loop with tqdm progress bar
+        for i in tqdm(range(n_batch)):
 
             # select a random training instance (X, y)
             if not sequential:
@@ -484,20 +634,22 @@ class ANN:
 
             # store the loss value
             if store_loss:
-                # l = 0.0
-                # for k in range(self.n_out):
-                # l += self.layers[-1].L_i
-
                 l = self.layers[-1].L_i
+                loss_i = np.mean(l)
+                self.loss_vals.append(loss_i)
 
                 if np.mod(i, 1000) == 0:
-                    loss_i = np.mean(l)
                     if verbose:
-                        print('Batch', i, 'learning rate', alpha, 'loss:', loss_i)
-                    # note: appending a cupy value to a list is inefficient -
-                    # if done every iteration
-                    # it will slow down executing significantly
-                    self.loss_vals.append(loss_i)
+                        # print('Batch', i, 'learning rate', alpha, 'loss:', loss_i)
+                        tqdm.write(' loss = %.4f' % (loss_i,))
+
+        if self.dropout:
+            # scale all weight matrices by dropout prob after training
+            for i in range(1, self.n_layers + 1):
+                self.layers[1].W *= self.dropout_prob[i - 1]
+
+            # turn off dropout after training
+            self.dropout = False
 
         if self.save:
             self.save_ANN()
@@ -630,12 +782,13 @@ class ANN:
             X = self.X
             y = self.y
         else:
-            print('Computing number of misclassifications wrt specified data (',
-                  X.shape[0], 'samples)')
+            print('Computing number of misclassifications wrt specified data, %d samples' % (y.size,))
 
         n_samples = X.shape[0]
-
-        for i in range(n_samples):
+        error_idx = []
+        
+        # loop with tqdm progress bar
+        for i in tqdm(range(n_samples)):
             _, max_idx_ann, _ = self.get_softmax(X[i].reshape([1, self.n_in]))
 
             max_idx_data = np.array([np.where(y_j == 1.0)[0]
@@ -644,15 +797,12 @@ class ANN:
             for j in range(self.n_softmax):
                 if max_idx_ann[j] != max_idx_data[j]:
                     n_misclass[j] += 1
-
-            if np.mod(i, 1000) == 0:
-                print('Computing misclassification error:',
-                      np.around(i / n_samples * 100, 1), '%')
+                    error_idx.append(i)
 
         print('Number of misclassifications =', n_misclass)
         print('Misclassification percentage =', n_misclass / n_samples * 100, '%')
 
-        return n_misclass / n_samples
+        return n_misclass / n_samples, error_idx
 
     def get_n_weights(self):
         """
@@ -691,8 +841,7 @@ class ANN:
         print('Loss function =', self.loss)
         print('Number of neurons per hidden layer =', self.n_neurons)
         print('Number of output neurons =', self.n_out)
-        print('Activation hidden layers =', self.activation)
-        print('Activation output layer =', self.activation_out)
+        print('Activation =', self.layer_activation)
         # print('On GPU =', self.on_gpu)
         self.get_n_weights()
         print('===============================')
