@@ -28,9 +28,12 @@ class ANN_Surrogate(Campaign):
     def train(self, feats, target, n_iter, lags=None, local=False,
               test_frac=0.0,
               n_layers=2, n_neurons=100,
+              loss='squared',
               activation='tanh',
+              learning_rate = 0.001, decay_rate = 0.9, beta1 = 0.9,
               batch_size=64, lamb=0.0,
-              standardize_X=True, standardize_y=True, **kwargs):
+              standardize_X=True, standardize_y=True,
+              dropout=False, **kwargs):
         """
         Perform back propagation to train the ANN
 
@@ -45,9 +48,20 @@ class ANN_Surrogate(Campaign):
         n_layers : The number of layers in the neural network. Includes the
                    input layer and the hidden layers. The default is 2.
         n_neurons : The number of neurons per layer. The default is 100.
+        loss : string, optional
+            The name of the loss function. The default is 'squared'.
         activation : Type of activation function. The default is 'tanh'.
+        learing_rate : the baseline learning rate. Is made parameter specific
+                       via RMSProp. Th edefault is 0.001.
+        decay_rate : float, optional
+                     Factor multiplying the decay rate every decay_step 
+                     iterations. Default is 0.9.
+        beta1 : float, optional
+                Momentum parameter controlling the moving average of the loss gradient.
+                Used for the parameter-specific learning rate. The default is 0.9.
         batch_size : Mini batch size. The default is 64.
         lamb : L2 regularization parameter. The default is 0.0.
+        dropout : Boolean flag for use of dropout regularization. 
 
         Returns
         -------
@@ -64,11 +78,12 @@ class ANN_Surrogate(Campaign):
         # test fraction
         self.test_frac = test_frac
 
+        # loss function
+        self.loss = loss
+
         # prepare the training data
         X_train, y_train, _, _ = self.feat_eng.get_training_data(
             feats, target, lags=lags, local=local, test_frac=test_frac, train_first=True)
-        # get the maximum lag that was specified
-        # TODO for 20 grid points returns (n_samples x n_features) as training dataset
         self.max_lag = self.feat_eng.max_lag
 
         # number of output neurons
@@ -78,23 +93,27 @@ class ANN_Surrogate(Campaign):
         self.neural_net = es.methods.ANN(X=X_train, y=y_train,
                                          n_layers=n_layers, n_neurons=n_neurons,
                                          n_out=n_out,
-                                         loss='squared',
+                                         loss=loss,
                                          activation=activation, batch_size=batch_size,
-                                         lamb=lamb, decay_step=10**4, decay_rate=0.9,
+                                         alpha=learning_rate,
+                                         lamb=lamb, decay_step=10**4, 
+                                         decay_rate=decay_rate, beta1=beta1,
                                          standardize_X=standardize_X,
                                          standardize_y=standardize_y,
-                                         save=False, **kwargs)
+                                         save=False,
+                                         **kwargs)
 
         print('===============================')
         print('Training Artificial Neural Network...')
 
         # train network for n_iter mini batches
-        self.neural_net.train(n_iter, store_loss=True)
+        self.neural_net.train(n_iter, store_loss=True, dropout=dropout,
+                              **kwargs)
         self.set_data_stats()
         if lags is not None:
             self.feat_eng.initial_condition_feature_history(feats)
 
-    def derivative(self, x, norm=True):
+    def derivative(self, x, norm=True, layer_idx=0):
         """
         Compute a derivative of the network output f(x) with respect to the inputs x.
 
@@ -107,6 +126,8 @@ class ANN_Surrogate(Campaign):
             Compute the gradient of ||f||_2. If False it computes the gradient of
             f, if f is a scalar. If False and f is a vector, the resulting gradient is of the
             column sum of the full Jacobian matrix.
+        layer_idx : int, optional, default is 0.
+            Index for the layer of which to return the derivative. Default is 0, the input layer.
 
         Returns
         -------
@@ -116,7 +137,7 @@ class ANN_Surrogate(Campaign):
         """
         # check that x is of shape (n_in, ) or (n_in, 1)
         assert x.shape[0] == self.neural_net.n_in, \
-        "x must be of shape (n_in,): %d != %d" % (x.shape[0], self.neural_net.n_in)
+            "x must be of shape (n_in,): %d != %d" % (x.shape[0], self.neural_net.n_in)
 
         if x.ndim > 1:
             assert x.shape[1] == 1, "Only pass 1 feature vector at a time"
@@ -129,7 +150,8 @@ class ANN_Surrogate(Campaign):
         x = (x - self.feat_mean) / self.feat_std
 
         # feed forward and compute and the derivatives
-        df_dx = self.neural_net.d_norm_y_dX(x.reshape([1, -1]), feed_forward=True, norm=norm)
+        df_dx = self.neural_net.d_norm_y_dX(x.reshape([1, -1]), feed_forward=True, norm=norm,
+                                            layer_idx=layer_idx)
 
         return df_dx
 
@@ -260,12 +282,16 @@ class ANN_Surrogate(Campaign):
 
         """
 
-        # features were standardized during training, do so here as well
+        # if features were standardized during training, do so here as well
         feat = (feat - self.feat_mean) / self.feat_std
-        # feed forward prediction step
-        y = self.neural_net.feed_forward(feat.reshape([1, self.neural_net.n_in])).flatten()
-        # transform y back to physical domain
-        y = y * self.output_std + self.output_mean
+        if self.loss == 'cross_entropy':
+            # y = the probability mass function at the output layer
+            y, _, _ = self.neural_net.get_softmax(feat.reshape([1, self.neural_net.n_in]))
+        else:
+            # feed forward prediction step
+            y = self.neural_net.feed_forward(feat.reshape([1, self.neural_net.n_in])).flatten()
+            # transform y back to physical domain
+            y = y * self.output_std + self.output_mean
 
         return y
 
